import { Action } from './action.js';
import { Actor } from './actor.js';

class Worker {
  constructor(tokenId) {
    this.tokenId = tokenId;
    this.stakeAmount = 0;
    this.available = true;
    this.assignedRequestId = null;
  }
}

/**
 * Purpose: Enable staking reputation to enter the pool of workers
 */
export class Availability extends Actor {
  workers = new Map();

  constructor(bench, name, scene) {
    super(name, scene);
    this.bench = bench;

    this.actions = {
      assignWork: new Action('assign work', scene),
    };
  }

  register({ stakeAmount, tokenId }) {
    // TODO: expire after duration
    // ? Is a particular stake amount required?
    const worker = this.workers.get(tokenId) ?? new Worker(tokenId);
    if (!worker.available) {
      throw new Error('Worker is already registered and busy. Can not increase stake.');
    }
    worker.stakeAmount += stakeAmount;
    // TODO: Interact with Bench contract to encumber reputation?
    this.workers.set(tokenId, worker);
  }

  // unregister() { }

  get availableWorkers() {
    return Array.from(this.workers.values()).filter(({ available }) => !!available);
  }

  async assignWork(requestId) {
    // Get random worker
    // TODO: Probability proportional to stakes
    const index = Math.floor(Math.random() * this.availableWorkers.length);
    const worker = this.availableWorkers[index];
    worker.available = false;
    worker.assignedRequestId = requestId;
    // TODO: Notify assignee
    return worker;
  }

  async getAssignedWork(reputationPublicKey) {
    const worker = this.workers.get(reputationPublicKey);
    return worker.assignedRequestId;
  }
}
